(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



BeginPackage["PhasePlot`"]; 


PhasePlot::usage="PhasePlot[odes, ic, {t, tstart, tend}, options}] returns a graphics object that is the phase portrait of the two-dimensional or three-dimensional system of odes.\nic is a list of ininitial condition points.\nt is the time variable in in odes.\ntstart, tend are the start and end times of the plot.\noptions may be any options for Plot.\n\nExample:\n\nPhasePlot[{u'[t]\[Equal] (1/4) u[t] - v[t], v'[t] \[Equal] u[t]},{{0, 1},  {0, .7}}, {t, 0,  50}, PlotRange \[Rule] {{-10, 10}, {-10, 10}}]";


Begin["`Private`"]; 


Filter[f_, options___?OptionQ]:= 
Sequence@@FilterRules[Flatten[{options}],Options[f]];



PhasePlot[odes_?ListQ,   ic_?ListQ, {t_, tstart_, tend_},
opt___?OptionQ]:= Module[{solver, SysIC, variables, varnames, variable, debug, dpr, icheck},

(* to turn on debugging set "debug"-> True *)

debug = "debug"/.{opt}/.{"debug"-> False}; 
dpr[u___]:= If[debug, Print["dbg: PhasePlot: ", u]]; 

(* check to make sure the system has dimension 2 *)

If[Length[odes]==3, Return[PhasePlot3D[odes, ic, {t, tstart, tend}, opt]]]; 
If[Length[odes]!= 2, 
Print["Error: PhasePlot: Expecting a system of two or three ODES. This system appears to have ", Length[odes], " ODES."]; Return[$Failed]; 
]; 

iccheck = And@@(ListQ/@ic); 
If[ Not[iccheck], Print["Error: PhasePlot: Initial Conditions should be a list of points."]; 
Return[$Failed]]; 
iccheck = Union[Length/@ic]; 
If[ Not[Length[iccheck]==1], Print["Error: PhasePlot: Initial Conditions should be a list of points of the form {{x1,y1}, {x2, y2}, ...} - at least one of the points does not have dimension 2."]; 
Return[$Failed]]; 
If[ iccheck[[1]]!= 2, Print["Error: PhasePlot: Initial Conditions should be a list of points in 2D."]; 
Return[$Failed]]; 


(* Determine the names of the variables, the functions to be plotted *)

variable[u_'[v_]]:= u[v]; 
variable[u___]:= (Print["Error: unable to determine variable: ", u]; Return[$Failed]); 
variables = variable/@(First/@Flatten[odes]); 
varnames = Head/@variables;

dpr["odes: ", odes];
 dpr["vars: ", variables];
dpr["varnames: ", varnames]; 

(* solver is a wrapper for NDSolve to solve the odes on a single curve from a given initial condition *)

solver[u_, v_]:=Module[{xvar, yvar, system}, 
{xvar, yvar} = varnames; 
system=Join[odes, {xvar[0]==u, yvar[0]==v}]; 
dpr["system: ", system]; 

variables/.NDSolve[
system,
variables,{t,tstart,tend}][[1]]
];
solver[{u_, v_}]:= solver[u, v];

(* calculate all the suolutions and plot them *) 

 With[{popts=Filter[Plot, opt]}, 
ParametricPlot[
Evaluate[solver/@ic],{t, tstart, tend}, 
DisplayFunction-> Identity, 
popts]
]
]


PhasePlot3D[odes_?ListQ,   ic_?ListQ, {t_, tstart_, tend_},
opt___?OptionQ]:= Module[{solver, SysIC, variables, varnames, variable, debug, dpr,  iccheck},

(* to turn on debugging set "debug"-> True *)

debug = "debug"/.{opt}/.{"debug"-> False}; 
dpr[u___]:= If[debug, Print["dbg: PhasePlot3D: ", u]]; 

(* check to make sure the system has dimension 3 *)

If[Length[odes]!= 3, 
Print["Error: PhasePlot: Expecting a system of two or three ODES. This system appears to have ", Length[odes], " ODES."]; Return[$Failed]; 
]; 

iccheck = And@@(ListQ/@ic); 
If[ Not[iccheck], Print["Error: PhasePlot: Initial Conditions should be a list of points."]; 
Return[$Failed]]; 
iccheck = Union[Length/@ic]; 
If[ Not[Length[iccheck]==1], Print["Error: PhasePlot: Initial Conditions should be a list of points of the form {{x1,y1, z1}, {x2, y2, z2}, ...} - at least one of the points does not have dimension 3."]; 
Return[$Failed]]; 
If[ iccheck[[1]]!= 3, Print["Error: PhasePlot: Initial Conditions should be a list of points in 3D."]; 
Return[$Failed]]; 

(* Determine the names of the variables, the functions to be plotted *)

variable[u_'[v_]]:= u[v]; 
variable[u___]:= (Print["Error: unable to determine variable: ", u]; Return[$Failed]); 
variables = variable/@(First/@Flatten[odes]); 
varnames = Head/@variables;

dpr["odes: ", odes];
 dpr["vars: ", variables];
dpr["varnames: ", varnames]; 

(* solver is a wrapper for NDSolve to solve the odes on a single curve from a given initial condition *)

solver[u_, v_, w_]:=Module[{xvar, yvar, system}, 
{xvar, yvar, zvar} = varnames; 
system=Join[odes, {xvar[0]==u, yvar[0]==v, zvar[0]==w}]; 
dpr["system: ", system]; 

variables/.NDSolve[
system,
variables,{t,tstart,tend}][[1]]
];
solver[{u_, v_, w_}]:= solver[u, v, w];

(* calculate all the suolutions and plot them *) 


 With[{popts=Filter[Plot, opt]}, 
ParametricPlot3D[
Evaluate[solver/@ic],{t, tstart, tend}, 
DisplayFunction-> Identity, 
popts]
]
]


End[];
EndPackage[]; 
